@@ -1,4 +1,4 @@
-# $Id: LaheySpace.pm,v 1.5 2004/10/25 18:06:53 jquelin Exp $
+# $Id: LaheySpace.pm,v 1.6 2005/12/02 20:47:33 jquelin Exp $
 #
 # Copyright (c) 2002-2003 Jerome Quelin <jquelin@cpan.org>
 # All rights reserved.
@@ -153,6 +153,27 @@ sub store_binary {
     return ($maxlen, 1 );
 }
 
+=head2 get_char( x, y )
+
+
+Return the character stored in the torus at the specified location. If
+the value is not between 0 and 255 (inclusive), get_char will return a
+string that looks like "<np-0x4500>".
+
+B</!\> As in Befunge, code and data share the same playfield, the
+character returned can be either an instruction B<or> raw data.  No
+guarantee is made that the return value is printable.
+
+=cut
+sub get_char {
+	my $self = shift;
+	my ($x,$y) = @_;
+	my $ord = $self->get_value($x,$y);
+	# reject invalid ascii
+	return sprintf("<np-0x%x>",$ord) if ($ord < 0 || $ord > 255);
+	return chr($ord);
+}
+
 =head2 get_value( x, y )
 
 
@@ -222,11 +243,12 @@ sub move_ip_forward {
     # Lahey-space, and if we need to wrap, we perform a
     # Lahey-space wrapping. That is, the play field is limited to
     # real code, and we do _not_ perform wrapping on all the
-    # addressable space.
-    $x = $self->{xmin} + ($x - $self->{xmax}) - 1 if $x > $self->{xmax};
-    $x = $self->{xmax} - ($x - $self->{xmin}) - 1 if $x < $self->{xmin};
-    $y = $self->{ymin} + ($y - $self->{ymax}) - 1 if $y > $self->{ymax};
-    $y = $self->{ymax} - ($y - $self->{ymin}) - 1 if $y < $self->{ymin};
+    # addressable space.  We use while loops in case the delta is
+    # larger than the torus width.
+    $x = $self->{xmin} + ($x - $self->{xmax}) - 1 while $x > $self->{xmax};
+    $x = $self->{xmax} - ($x - $self->{xmin}) - 1 while $x < $self->{xmin};
+    $y = $self->{ymin} + ($y - $self->{ymax}) - 1 while $y > $self->{ymax};
+    $y = $self->{ymax} - ($y - $self->{ymin}) - 1 while $y < $self->{ymin};
 
     # Store new position.
     $ip->set_pos( $x, $y );
